// Library     : DPMI
// Description : DOS Protected Mode Interface function library
// Module      : DPMI00.C
// Version     : 1.00 - 01/09/1994
// Written by  : David A. Mair
// -----------------------------------------------------------------------------
// Change summary --------------------------------------------------------------
// -----------------------------------------------------------------------------
//    Date   | Version | Comment and author
// -----------------------------------------------------------------------------
//           |         |
//           |         |
// -----------------------------------------------------------------------------

// Inline assembler is included in this module
#pragma inline

#include "dpmi.h"


// Release Current Virtual Machine's Time Slice
BOOL ReleaseTimeSlice(void)
{
	BOOL bRetVal;
	
	asm {
		// Call the DPMI function
		mov		ax, 0x1680
		int		0x2F

		// Convert al to a word in ax
		cbw

#ifdef __DPMI32__
		// When 32 bit compile bRetval is a long
		cwd
		mov		dword ptr [bRetVal], eax
#else
		// Store return value
		mov		word ptr [bRetVal], ax
#endif
	}

	return bRetVal;
}


BOOL GetCPUMode(void)
{
	BOOL bRetVal;

	asm {
		// Call the DPMI function
		mov		ax, 0x1686
		int		0x2F

#ifdef __DPMI32__
		// 32 bit compile, so we return a 32 bit BOOL
		cwd
		mov		dword ptr [bRetVal], eax
#else
		// Store the return value
		mov		word ptr [bRetVal], ax
#endif
	}

	return bRetVal;
}


#ifdef __DPMI32__
	WORD AllocateLDTDescriptors(WORD wNumberToAllocate, SELECTOR *fpBaseSelector)
#else
	WORD AllocateLDTDescriptors(WORD wNumberToAllocate, SELECTOR far *fpBaseSelector)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0000
		mov		cx, word ptr [wNumberToAllocate]
		int		0x31

		jc		aldtdSetRetVal

		push	es
#ifdef __DPMI32__
		mov		di, ds
		mov		es, di
		mov		edi, dword ptr [fpBaseSelector]
		mov		word ptr es:[edi], ax
#else
		les		di, fpBaseSelector
		mov		word ptr es:[di], ax
#endif
		pop		es

		mov		ax, 0x0000
	}

aldtdSetRetVal:
	asm {	
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


WORD FreeLDTDescriptor(SELECTOR SelectorToFree)
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0001
		mov		bx, word ptr [SelectorToFree]
		int		0x31

		jc		fldtdSetRetVal

		mov		ax, 0x0000
	}

fldtdSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
	WORD SegmentToDescriptor(SEGMENT RealModeSegment, SELECTOR *fpSelectorForSegment)
#else
	WORD SegmentToDescriptor(SEGMENT RealModeSegment, SELECTOR far *fpSelectorForSegment)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0002
		mov		bx, word ptr [RealModeSegment]
		int		0x31

		jc		stdSetRetVal

		push	es
#ifdef __DPMI32__
		mov		ax, ds
		mov		es, ax
		mov		edi, dword ptr [fpSelectorForSegment]
		mov		word ptr es:[edi], ax
#else
		les		di, fpSelectorForSegment
		mov		word ptr es:[di], ax
#endif
		pop		es

		mov		ax, 0x0000
	}

stdSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


WORD GetSelectorIncrementValue(void)
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0003
		int		0x31

		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
	WORD GetSegmentBaseAddress(SELECTOR Selector, DWORD *fpdwBaseAddress)
#else
	WORD GetSegmentBaseAddress(SELECTOR Selector, DWORD far *fpdwBaseAddress)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0006
		mov		bx, word ptr [Selector]
		int		0x31

		jc		gsbaSetRetVal

		push	es
#ifdef __DPMI32__
		mov		ax, ds
		mov		es, ax
		mov		edi, dword ptr [fpdwBaseAddress]
		mov		word ptr es:[edi], dx
		mov		word ptr es:[edi + 2], cx
#else
		les		di, fpdwBaseAddress
		mov		word ptr es:[di], dx
		mov		word ptr es:[di + 2], cx
#endif
		pop		es

		mov		ax, 0x0000
	}

gsbaSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


WORD SetSegmentBaseAddress(SELECTOR Selector, DWORD dwBaseAddress)
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0007
		mov		bx, word ptr [Selector]
		mov		dx, word ptr [dwBaseAddress]
		mov		cx, word ptr [dwBaseAddress + 2]
		int		0x31

		jc		ssbaSetRetVal

		mov		ax, 0x0000
	}

ssbaSetRetVal:
	asm {

		mov		wRetVal, ax
	}

	return wRetVal;
}


DWORD GetSegmentLimit(SELECTOR Selector)
{
	DWORD dwRetVal;

	asm {
#ifdef __DPMI32__
		movzx	ebx, word ptr [Selector]
		lsl		eax, ebx
		mov		dword ptr [dwRetVal], eax
#else
		lsl		ax, word ptr [Selector]
		mov		word ptr[dwRetVal], ax
		mov		word ptr [dwRetVal + 2], 0x0000
#endif
	}

	return dwRetVal;
}


WORD SetSegmentLimit(SELECTOR Selector, DWORD dwLimit)
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0008
		mov		bx, word ptr [Selector]
		mov		dx, word ptr [dwLimit]
		mov		cx, word ptr [dwLimit + 2]
		int		0x31

		jc		sslSetRetVal

		mov		ax, 0x0000
	}

sslSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
ACCESS_RIGHTS GetDescriptorAccessRights(SELECTOR Selector, EXTENDED_ACCESS_RIGHTS *byExtendedRights)
#else
ACCESS_RIGHTS GetDescriptorAccessRights(SELECTOR Selector, EXTENDED_ACCESS_RIGHTS far *byExtendedRights)
#endif
{
	ACCESS_RIGHTS byRetVal;
	EXTENDED_ACCESS_RIGHTS byExtRights;

	asm {
#ifdef __DPMI32__
		movzx	ebx, word ptr [Selector]
		lar		eax, ebx
		mov		byte ptr [byRetVal], ah
		shr		eax, 8
		mov		byte ptr [byExtRights], ah
#else
		lar		ax, word ptr [Selector]
		mov		byte ptr [byRetVal], ah
		mov		byte ptr [byExtRights], 0x0000
#endif
	}

	if (byExtendedRights != NULL)
	{
		*byExtendedRights = byExtRights;
	}

	return byRetVal;
}

WORD SetDescriptorAccessRights(SELECTOR Selector, ACCESS_RIGHTS AccessRights, EXTENDED_ACCESS_RIGHTS ExtendedRights)
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0009
		mov		bx, word ptr [Selector]
		mov		cl, byte ptr [AccessRights]
		mov		ch, byte ptr [ExtendedRights]
		int		0x31

		jc		sdarSetRetVal

		mov		ax, 0x0000
	}

sdarSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
	WORD CreateAliasDescriptor(SELECTOR Source, SELECTOR *fpAlias)
#else
	WORD CreateAliasDescriptor(SELECTOR Source, SELECTOR far *fpAlias)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x000A
		mov		bx, word ptr [Source]
		int		0x31

		jc		cadSetRetVal

		push	es
#ifdef __DPMI32__
		mov		di, ds
		mov		es, di
		mov		edi, dword ptr [fpAlias]
		mov		word ptr es:[edi], ax
#else
		les		di, fpAlias
		mov		word ptr es:[di], ax
#endif
		pop		es

		mov		ax, 0x0000
	}

cadSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
	WORD GetDescriptor(SELECTOR Selector, DESCRIPTOR *fpDescriptor)        
#else
	WORD GetDescriptor(SELECTOR Selector, DESCRIPTOR far *fpDescriptor)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x000B
		mov		bx, word ptr [Selector]
		push	es
#ifdef __DPMI32__
		mov		cx, ds
		mov		es, cx
		mov		edi, dword ptr [fpDescriptor]
#else
		les		di, fpDescriptor
#endif
		int		0x31

		pop		es
		jc		gdSetRetVal

		mov		ax, 0x0000
	}

gdSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


WORD SetDescriptor(SELECTOR Selector, DESCRIPTOR Descriptor)
{
	WORD wRetVal;

	asm {
		mov		ax, 0x000C
		mov		bx, word ptr [Selector]
		push	es
		mov		di, ss
		mov		es, di
#ifdef __DPMI32__
		lea		edi, Descriptor
#else
		lea		di, Descriptor
#endif
		int		0x31
		
		pop		es

		jc		sdSetRetVal

		mov		ax, 0x0000
	}

sdSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


WORD AllocateSpecificLDTDescriptor(SELECTOR Selector)
{
	WORD wRetVal;

	asm {
		mov		ax, 0x000D
		mov		bx, word ptr [Selector]
		int		0x31

		jc		asldtdSetRetVal

		mov		ax, 0x0000
	}

asldtdSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
	WORD GetMultipleDescriptors(WORD *wNumberOfDescriptors, DESCRIPTOR_LIST *fpDescriptorList)
#else
	WORD GetMultipleDescriptors(WORD far *wNumberOfDescriptors, DESCRIPTOR_LIST far *fpDescriptorList)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x000E
		push	es
#ifdef __DPMI32__
		mov		edi, dword ptr [wNumberOfDescriptors]
		mov		cx, word ptr ds:[edi]
		mov		bx, ds
		mov		es, bx
		mov		edi, dword ptr [fpDescriptorList]
#else
		les		di, wNumberOfDescriptors
		mov		cx, word ptr es:[di]
		les		di, fpDescriptorList
#endif
		int		0x31

		pop		es

		jc		gmdSetRetVal

		mov		word ptr [wRetVal], 0x0000
		jmp		gmdAllDone
	}

gmdSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
#ifdef __DPMI32__
		mov		edi, dword ptr [wNumberOfDescriptors]
		mov		word ptr ds:[edi], cx
#else
		push	es
		les		di, wNumberOfDescriptors
		mov		word ptr es:[di], cx
		pop		es
#endif
	}

gmdAllDone:
	return wRetVal;
}


#ifdef __DPMI32__
	WORD SetMultipleDescriptors(WORD *wNumberOfDescriptors, DESCRIPTOR_LIST *fpDescriptorList)
#else
	WORD SetMultipleDescriptors(WORD far *wNumberOfDescriptors, DESCRIPTOR_LIST far *fpDescriptorList)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x000F
		push	es
#ifdef __DPMI32__
		mov		edi, dword ptr [wNumberOfDescriptors]
		mov		cx, word ptr ds:[edi]
		mov		bx, ds
		mov		es, bx
		mov		edi, dword ptr [fpDescriptorList]
#else
		les		di, wNumberOfDescriptors
		mov		cx, word ptr es:[di]
		les		di, fpDescriptorList
#endif
		int		0x31

		pop		es

		jc		smdSetRetVal

		mov		word ptr [wRetVal], 0x0000
		jmp		smdAllDone
	}

smdSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
#ifdef __DPMI32__
		mov		edi, dword ptr [wNumberOfDescriptors]
		mov		word ptr ds:[edi], cx
#else
		push	es
		les		di, wNumberOfDescriptors
		mov		word ptr es:[di], cx
		pop		es
#endif
	}

smdAllDone:
	return wRetVal;
}

